/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.reporter;



import java.util.List;
import java.util.Map;



/**
 * Class representing test results read from log file(s).
 * 
 * @author Pavel Tisnovsky
 */
public class TestResult
{
    /**
     * Number of all tests.
     */
    private int total;

    /**
     * Number of passed tests.
     */
    private int passed;

    /**
     *  Number of failed tests.
     */
    private int failed;

    /**
     * Number of tests which were not finished due to other error(s).
     */
    private int error;

    /**
     * Number of tests which were ignored because they are prepared for a different version of Thermostat.
     */
    private int ignored;

    /**
     * Date when tests were run.
     */
    private String date;

    /**
     * Constructor which initializes all attributes.
     */
    public TestResult()
    {
        this.setTotal(0);
        this.setPassed(0);
        this.setFailed(0);
        this.setError(0);
        this.setIgnored(0);
        this.setDate(null);
    }

    /**
     * Constructor which initializes all attributes but date.
     */
    public TestResult(int passed, int failed, int error, int ignored)
    {
        this.setTotal(passed + failed + error + ignored);
        this.setPassed(passed);
        this.setFailed(failed);
        this.setError(error);
        this.setIgnored(ignored);
        this.setDate(null);
    }

    /**
     * Setter for the "total" attribute.
     *
     * @param passed
     *            the passed to set
     */
    public void setTotal(int total)
    {
        this.total = total;
    }

    /**
     * Getter for the "total" attribute.
     *
     * @return the total value
     */
    public int getTotal()
    {
        return this.total;
    }

    /**
     * Setter for the "passed" attribute.
     *
     * @param passed
     *            the passed to set
     */
    public void setPassed(int passed)
    {
        this.passed = passed;
    }

    /**
     * Getter for the "passed" attribute.
     *
     * @return the passed
     */
    public int getPassed()
    {
        return this.passed;
    }

    /**
     *  Setter for the "failed" attribute.
     *
     * @param failed
     *            the failed to set
     */
    public void setFailed(int failed)
    {
        this.failed = failed;
    }

    /**
     * Getter for the "failed" attribute.
     *
     * @return the failed
     */
    public int getFailed()
    {
        return this.failed;
    }

    /**
     *  Setter for the "error" attribute.
     *
     * @param error
     *            the error to set
     */
    public void setError(int error)
    {
        this.error = error;
    }

    /**
     * Getter for the "error" attribute.
     *
     * @return the error
     */
    public int getError()
    {
        return this.error;
    }

    /**
     *  Setter for the "ignored" attribute.
     *
     * @param error
     *            the error to set
     */
    public void setIgnored(int ignored)
    {
        this.ignored = ignored;
    }

    /**
     * Getter for the "ignored" attribute.
     *
     * @return the error
     */
    public int getIgnored()
    {
        return this.ignored;
    }

    /**
     * Setter for a "date" attribute.
     *
     * @param date
     *            the date to set
     */
    public void setDate(String date)
    {
        this.date = date;
    }

    /**
     * Getter for the "date" attribute.
     *
     * @return the date
     */
    public String getDate()
    {
        return this.date;
    }

    /**
     * Returns name of operation system where reporter (and tests) are running.
     * 
     * @return name of operation system where reporter (and tests) are running
     */
    public String getOsName()
    {
        return System.getProperty("os.name");
    }

    /**
     * Returns version of operation system where reporter (and tests) are running.
     * 
     * @return version of operation system where reporter (and tests) are running
     */
    public String getOsVer()
    {
        return System.getProperty("os.version");
    }

    /**
     * Returns architecture where reporter (and tests) are running.
     * 
     * @return architecture where reporter (and tests) are running
     */
    public String getOsArch()
    {
        return System.getProperty("os.arch");
    }

    /**
     * Returns Java version where reporter (and tests) are running.
     * 
     * @return Java version where reporter (and tests) are running
     */
    public String getJavaVersion()
    {
        return System.getProperty("java.version");
    }

    /**
     * Returns JVM name where reporter (and tests) are running.
     * 
     * @return JVM name where reporter (and tests) are running
     */
    public String getVmName()
    {
        return System.getProperty("java.vm.name");
    }

    /**
     * Returns JVM version where reporter (and tests) are running.
     * 
     * @return JVM version where reporter (and tests) are running
     */
    public String getVmVersion()
    {
        return System.getProperty("java.vm.version");
    }

    /**
     * Returns the URL where the results are about to be stored.
     * This URL is stored in the environment variable RESULTS_URL.
     * 
     * @return URL where the results are about to be stored.
     */
    public String getResultsUrl()
    {
        String resultsURL = System.getenv().get("RESULTS_URL");
        return resultsURL != null ? resultsURL : "not set";
    }

    /**
     * Read test summary from a log file and fill in all attributes.
     *
     * @param testResultForDate log file content
     *
     * @return instance of this class filed with all attributes.
     */
    public static TestResult readSummary(Map<String, List<String>> testResultForDate)
    {
        TestResult testResult = new TestResult();
        // iterate over all contents
        for (Map.Entry<String, List<String>> testResultContent : testResultForDate.entrySet())
        {
            for (String logLine : testResultContent.getValue())
            {
                // we are interested only with lines beginning with "SUMMARY: "
                if (logLine.startsWith("SUMMARY: "))
                {
                    final String strTotalTests = StringUtils.getSubstring(logLine, "total: ", "passed: ");
                    final String strPassedTests = StringUtils.getSubstring(logLine, "passed: ", "failed: ");
                    final String strFailedTests = StringUtils.getSubstring(logLine, "failed: ", "error: ");
                    final String strErrorTests = StringUtils.getSubstring(logLine, "error: ", "ignored: ");
                    final String strIgnoredTests = StringUtils.getSubstring(logLine, "ignored: ", "duration: ");
                    final int totalTests = Integer.parseInt(strTotalTests);
                    final int passedTests = Integer.parseInt(strPassedTests);
                    final int failedTests = Integer.parseInt(strFailedTests);
                    final int errorTests = Integer.parseInt(strErrorTests);
                    final int ignoredTests = Integer.parseInt(strIgnoredTests);
                    testResult.setTotal(testResult.getTotal() + totalTests);
                    testResult.setPassed(testResult.getPassed() + passedTests);
                    testResult.setFailed(testResult.getFailed() + failedTests);
                    testResult.setError(testResult.getError() + errorTests);
                    testResult.setIgnored(testResult.getIgnored() + ignoredTests);
                }
            }
        }
        return testResult;
    }

}
